{% extends "admin/base.html" %}
{% load i18n admin_tools_menu_tags static %}

{% block title %}{% trans 'Admin' %}{% endblock %}

{% block extrastyle %}
    {{ block.super }}
    {% if user.is_active and user.is_staff %}
        {% if not is_popup %}
            {% admin_tools_render_menu_css %}
        {% endif %}
    {% endif %}
{% endblock %}


{% block nav-global %}
    {% if user.is_active and user.is_staff %}
        {% if not is_popup %}
            {% admin_tools_render_menu %}
        {% endif %}
    {% endif %}
{% endblock %}
