#!/usr/bin/python
# -*- coding: utf-8 -*-

#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright (C) 2011-2014 Star2Billing S.L.
#
# The Initial Developer of the Original Code is
# Arezqui Belaid <info@star2billing.com>
#


__version__ = '0.24.0'  # edit also docs/source/conf.py and update requirements.txt
__author__ = "Petr Dlouhy"
__contact__ = "petr.dlouhy@emai.cz"
__homepage__ = "https://github.com/PetrDlouhy"
__docformat__ = "restructuredtext"
