{% extends "admin_tools/dashboard/module.html" %}
{% load nvd3_tags %}

{% block module_content %}
    {% if module.require_chart_jscss %}
       {% include '../../include_nvd3.html' %}
    {% endif %}

    {% if module.form_field %}
        <form class="stateform" action="." method="POST" enctype="multipart/form-data">{% csrf_token %}
            {{ module.form_field }}
        </form>
        <br/>
    {% endif %}

    {% include_container module.chart_container %}

{% endblock %}

